# frozen_string_literal: true

require 'securerandom'
require 'down'

module GsImgFetcher
  class Fetcher
    INITIALIZED = :initialized
    FETCHED = :fetched
    FETCH_FAILED = :fetch_failed
    SAVED = :saved
    SAVE_FAILED = :save_failed

    attr_reader :state, :uuid

    def initialize(entry, output_dir, max_size: nil)
      @entry = entry
      @output_dir = output_dir
      @max_size = max_size.yield_self { |s| s.present? ? s * 1024 * 1024 : nil }
      @uuid = SecureRandom.uuid
      @state = INITIALIZED
    end

    def fetch
      @tempfile = Down.download(@entry.url, **{ max_size: @max_size }.compact)
      @state = FETCHED
      log_fetched
    rescue Down::Error => e
      @state = FETCH_FAILED
      log_error(e)
    end

    def save
      return unless fetched?

      FileUtils.mkdir_p(@output_dir)
      FileUtils.mv(tempfile.path, output_path)
      @state = SAVED
      log_saved
    rescue StandardError => e
      @state = SAVE_FAILED
      log_error(e)
    end

    def output_path
      @output_path ||= File.join(@output_dir, [uuid, @entry.extension].join('.'))
    end

    def failed?
      [FETCH_FAILED, SAVE_FAILED].include?(state)
    end

    def fetched?
      FETCHED == state
    end

    def successful?
      SAVED == state
    end

    private

    attr_reader :tempfile

    def log_fetched
      LOGGER.info("Successfully fetched #{@entry.url}")
    end

    def log_saved
      LOGGER.info("Saved #{@entry.url} to #{output_path}")
    end

    def log_error(err)
      LOGGER.warn(err.message)
    end
  end
end
