# frozen_string_literal: true

require 'concurrent'

module GsImgFetcher
  class Manager
    class << self
      def fetch(input_path, **opts)
        entry_set = EntrySet.from_file(input_path)
        new(entry_set, **opts).setup.fetch
      end
    end

    attr_reader :entry_set, :entries, :successful_fetches, :failed_fetches

    DEFAULT_MAX_THREADS = Concurrent.processor_count

    def initialize(entry_set, output_dir: nil, async: true, max_threads: nil, max_size: nil)
      @entry_set = entry_set
      @output_dir = output_dir || Dir.pwd
      @async = async
      @max_threads = max_threads || DEFAULT_MAX_THREADS
      @max_size = max_size
      @entries = Queue.new
      @successful_fetches = Queue.new
      @failed_fetches = Queue.new
    end

    def setup
      tap do
        log_entries

        next unless entries.empty?

        entry_set.valid_entries.each { |e| entries.push(e) }
      end
    end

    def fetch
      @async ? async_fetch : sync_fetch

      log_result
      successful_fetches.close
      failed_fetches.close
    end

    private

    def async_fetch
      with_thread_pool do |pool|
        pool.post { fetch_one(entries.pop, @output_dir) } until entries.empty?
        entries.close
      end
    end

    def sync_fetch
      fetch_one(entries.pop, @output_dir) until entries.empty?
      entries.close
    end

    def with_thread_pool
      Concurrent::FixedThreadPool.new([entries.size, @max_threads].min).tap do |pool|
        yield(pool)
        pool.shutdown
        pool.wait_for_termination
      end
    end

    def fetch_one(entry, dir)
      Fetcher.new(entry, dir, max_size: @max_size).tap do |fetcher|
        fetcher.fetch && fetcher.save

        if fetcher.successful?
          successful_fetches.push(fetcher)
        else
          failed_fetches.push(fetcher)
        end
      end
    end

    def log_entries
      LOGGER.info("Processing #{entry_set.entries.count} URLs (#{entry_set.valid_entries.count} valid, #{entry_set.invalid_entries.count} invalid)")
    end

    def log_result
      LOGGER.info("Fetch complete (#{successful_fetches.size} successful, #{failed_fetches.size} failed)")
    end
  end
end
